@@ -1,14 +1,15 @@
 #ifndef BOOKMARKSDIALOG_H
 #define BOOKMARKSDIALOG_H
 
-#include <QDialog>
+#include <QMainWindow>
 
+class QCloseEvent;
 class Book;
 class QListWidget;
 class QListWidgetItem;
 
 /** Dialog box managing bookmarks. */
-class BookmarksDialog: public QDialog
+class BookmarksDialog: public QMainWindow
 {
     Q_OBJECT
 
@@ -16,13 +17,18 @@ public:
     explicit BookmarksDialog(Book *book, QWidget *parent = 0);
 
 signals:
+    void goToBookmark(int index);
+    void addBookmark();
 
 public slots:
     void onGo();
     void onAdd();
     void onItemActivated(QListWidgetItem *);
+    void onClose();
+    void onDelete(bool really = false);
 
 protected:
+    void closeEvent(QCloseEvent *e);
     Book *book;
     QListWidget *list;
 };
