@@ -2,6 +2,8 @@
 #define MAINWINDOW_H
 
 #include <QtGui>
+
+#include "bookwindow.h"
 #include "bookfinder.h"
 
 class QString;
@@ -10,8 +12,9 @@ class DevTools;
 class BookView;
 class Book;
 class FullScreenWindow;
+class Progress;
 
-class MainWindow: public QMainWindow
+class MainWindow: public BookWindow
 {
     Q_OBJECT
 
@@ -29,16 +32,17 @@ public slots:
     void showRegular();
     void showBig();
     void onSettingsChanged(const QString &key);
-    void onChapterLoadStart();
-    void onChapterLoadEnd(int index);
+    void onPartLoadStart();
+    void onPartLoadEnd(int index);
     void onAddBookmark();
     void onGoToBookmark(int index);
     void showChapters();
     void onGoToChapter(int index);
 
 protected:
-    virtual void closeEvent(QCloseEvent *event);
-    virtual void timerEvent(QTimerEvent *event);
+    void closeEvent(QCloseEvent *event);
+    void timerEvent(QTimerEvent *event);
+    void resizeEvent(QResizeEvent *event);
 
 private:
     void setCurrentBook(const QModelIndex &current);
@@ -64,6 +68,7 @@ private:
     int preventBlankingTimer;
     BookFinder *bookFinder;
     BookFinderThread bookFinderThread;
+    Progress *progress;
 };
 
 #endif // MAINWINDOW_H
