@@ -22,12 +22,13 @@ const int COVER_HEIGHT = 59;
 Book::Book(const QString &p, QObject *parent): QObject(parent)
 {
     mPath = "";
-    if (p != "") {
+    if (p.size()) {
         QFileInfo info(p);
         mPath = info.absoluteFilePath();
         title = info.baseName();
         cover = QImage(":/icons/book.png").scaled(COVER_WIDTH, COVER_HEIGHT,
-            Qt::IgnoreAspectRatio, Qt::SmoothTransformation);
+            Qt::KeepAspectRatioByExpanding, Qt::SmoothTransformation).
+            scaled(COVER_WIDTH, COVER_HEIGHT, Qt::KeepAspectRatio);
     }
 }
 
@@ -42,7 +43,7 @@ bool Book::open()
     t.trace(path());
     close();
     clear();
-    if (path() == "") {
+    if (path().isEmpty()) {
         title = "No book";
         return false;
     }
@@ -61,7 +62,7 @@ void Book::close()
 {
     Trace t("Book::close");
     content.clear();
-    toc.clear();
+    parts.clear();
     QDir::setCurrent(QDir::rootPath());
     clearDir(tmpDir());
 }
@@ -136,8 +137,8 @@ bool Book::parse()
     delete source;
 
     // Initially, put all content items in the chapter list.
-    // This can be refined by parsing the NCX file later
-    chapters = toc;
+    // This will be refined by parsing the NCX file later
+    chapters = parts;
 
     // Load cover image
     QStringList coverKeys;
@@ -145,8 +146,9 @@ bool Book::parse()
     foreach (QString key, coverKeys) {
         if (content.contains(key)) {
             t.trace("Loading cover image from " + content[key].href);
-            cover = QImage(content[key].href).scaled(COVER_WIDTH,
-                COVER_HEIGHT, Qt::IgnoreAspectRatio, Qt::SmoothTransformation);
+            cover = QImage(content[key].href).scaled(COVER_WIDTH, COVER_HEIGHT,
+                Qt::KeepAspectRatioByExpanding, Qt::SmoothTransformation).
+                scaled(COVER_WIDTH, COVER_HEIGHT, Qt::KeepAspectRatio);
             break;
         }
     }
@@ -168,6 +170,15 @@ bool Book::parse()
         delete source;
     }
 
+    // Calculate book part sizes
+    size = 0;
+    foreach (QString part, parts) {
+        QFileInfo info(content[part].href);
+        content[part].size = info.size();
+        size += content[part].size;
+        t.trace(QString("Size of part %1: %2").arg(part).arg(content[part].size));
+    }
+
     return ret;
 }
 
@@ -232,25 +243,26 @@ void Book::load()
     subject = settings.value(key + "subject").toString();
     source = settings.value(key + "source").toString();
     rights = settings.value(key + "rights").toString();
-    mLastBookmark.chapter = settings.value(key + "lastchapter").toInt();
+    mLastBookmark.part = settings.value(key + "lastpart").toInt();
     mLastBookmark.pos = settings.value(key + "lastpos").toReal();
     cover = settings.value(key + "cover").value<QImage>().scaled(COVER_WIDTH,
         COVER_HEIGHT, Qt::IgnoreAspectRatio, Qt::SmoothTransformation);
     if (cover.isNull()) {
         cover = QImage(":/icons/book.png").scaled(COVER_WIDTH, COVER_HEIGHT,
-            Qt::IgnoreAspectRatio, Qt::SmoothTransformation);
+            Qt::KeepAspectRatioByExpanding, Qt::SmoothTransformation).
+            scaled(COVER_WIDTH, COVER_HEIGHT, Qt::KeepAspectRatio);
     }
 
     // Load bookmarks
     int size = settings.value(key + "bookmarks").toInt();
     for (int i = 0; i < size; i++) {
-        int chapter = settings.value(key + "bookmark" + QString::number(i) +
-                                     "/chapter").toInt();
+        int part = settings.value(key + "bookmark" + QString::number(i) +
+                                     "/part").toInt();
         qreal pos = settings.value(key + "bookmark" + QString::number(i) +
                                    "/pos").toReal();
-        t.trace(QString("Bookmark %1 at chapter %2, %3").
-                arg(i).arg(chapter).arg(pos));
-        mBookmarks.append(Bookmark(chapter, pos));
+        t.trace(QString("Bookmark %1 at part %2, %3").
+                arg(i).arg(part).arg(pos));
+        mBookmarks.append(Bookmark(part, pos));
     }
 }
 
@@ -271,7 +283,7 @@ void Book::save()
     settings.setValue(key + "subject", subject);
     settings.setValue(key + "source", source);
     settings.setValue(key + "rights", rights);
-    settings.setValue(key + "lastchapter", mLastBookmark.chapter);
+    settings.setValue(key + "lastpart", mLastBookmark.part);
     settings.setValue(key + "lastpos", mLastBookmark.pos);
     settings.setValue(key + "cover", cover);
 
@@ -279,17 +291,17 @@ void Book::save()
     settings.setValue(key + "bookmarks", mBookmarks.size());
     for (int i = 0; i < mBookmarks.size(); i++) {
         t.trace(QString("Bookmark %1 at %2, %3").
-                arg(i).arg(mBookmarks[i].chapter).arg(mBookmarks[i].pos));
-        settings.setValue(key + "bookmark" + QString::number(i) + "/chapter",
-                          mBookmarks[i].chapter);
+                arg(i).arg(mBookmarks[i].part).arg(mBookmarks[i].pos));
+        settings.setValue(key + "bookmark" + QString::number(i) + "/part",
+                          mBookmarks[i].part);
         settings.setValue(key + "bookmark" + QString::number(i) + "/pos",
                           mBookmarks[i].pos);
     }
 }
 
-void Book::setLastBookmark(int chapter, qreal position)
+void Book::setLastBookmark(int part, qreal position)
 {
-    mLastBookmark.chapter = chapter;
+    mLastBookmark.part = part;
     mLastBookmark.pos = position;
     save();
 }
@@ -299,9 +311,9 @@ Book::Bookmark Book::lastBookmark() const
     return Book::Bookmark(mLastBookmark);
 }
 
-void Book::addBookmark(int chapter, qreal position)
+void Book::addBookmark(int part, qreal position)
 {
-    mBookmarks.append(Bookmark(chapter, position));
+    mBookmarks.append(Bookmark(part, position));
     qSort(mBookmarks.begin(), mBookmarks.end());
     save();
 }
@@ -349,7 +361,7 @@ QString Book::rootPath() const
 
 QString Book::name() const
 {
-    if (title != "") {
+    if (title.size()) {
         QString ret = title;
         if (creators.length()) {
             ret += "\nBy " + creators[0];
@@ -365,9 +377,66 @@ QString Book::name() const
 
 QString Book::shortName() const
 {
-    if (title == "") {
-        return QFileInfo(path()).baseName();
-    } else {
-        return title;
+    return (title.isEmpty())? QFileInfo(path()).baseName(): title;
+}
+
+int Book::chapterFromPart(int index)
+{
+    // FIXME
+    Q_UNUSED(index);
+    int ret = -1;
+    return ret;
+}
+
+int Book::partFromChapter(int index)
+{
+    Trace t("Book::partFromChapter");
+    QString id = chapters[index];
+    QString href = content[id].href;
+    QString baseRef(href);
+    QUrl url(QString("file://") + href);
+    if (url.hasFragment()) {
+        QString fragment = url.fragment();
+        baseRef.chop(fragment.length() + 1);
     }
+
+    // Swipe through all content items to find the one matching the chapter href
+    // FIXME: Do we need to index content items by href, too?
+    QString contentKey;
+    bool found = false;
+    foreach (contentKey, content.keys()) {
+        if (contentKey == id) {
+            continue;
+        }
+        if (content[contentKey].href == baseRef) {
+            found = true;
+            t.trace(QString("Key for %1 is %2").arg(baseRef).arg(contentKey));
+            break;
+        }
+    }
+    if (!found) {
+        t.trace("Could not find key for " + baseRef);
+        return -1;
+    }
+    int partIndex = parts.indexOf(contentKey);
+    if (partIndex == -1) {
+        qCritical()
+            << "Book::partFromChapter: Could not find part index of chapter"
+            << id;
+    }
+    return partIndex;
+}
+
+qreal Book::getProgress(int part, qreal position)
+{
+    Q_ASSERT(part < parts.size());
+    QString key;
+    qreal partSize = 0;
+    for (int i = 0; i < part; i++) {
+        key = parts[i];
+        partSize += content[key].size;
+    }
+    key = parts[part];
+    partSize += content[key].size * position;
+    return partSize / (qreal)size;
 }
