@@ -20,21 +20,18 @@ public:
     {
         QString href;
         QString name;
+        qint64 size;
     };
 
     /** Bookmark: a volume index and a relative position in volume. */
     struct Bookmark
     {
-        Bookmark(int chapter_, qreal pos_): chapter(chapter_), pos(pos_) {}
-        Bookmark() {chapter = pos = 0;}
-        int chapter;
+        Bookmark(int part_, qreal pos_): part(part_), pos(pos_) {}
+        Bookmark() {part = pos = 0;}
+        int part;
         qreal pos;
         bool operator<(const Bookmark&other) const {
-            if (chapter != other.chapter) {
-                return chapter < other.chapter;
-            } else {
-                return pos < other.pos;
-            }
+            return (part == other.part)? (pos < other.pos): (part < other.part);
         }
     };
 
@@ -72,13 +69,13 @@ public:
     bool clearDir(const QString &directory);
 
     /** Set last bookmark. */
-    void setLastBookmark(int chapter, qreal position);
+    void setLastBookmark(int part, qreal position);
 
     /** Get last bookmark. */
     Bookmark lastBookmark() const;
 
     /** Add bookmark. */
-    void addBookmark(int chapter, qreal position);
+    void addBookmark(int part, qreal position);
 
     /** Delete bookmark. */
     void deleteBookmark(int index);
@@ -95,8 +92,17 @@ public:
     /** Get short friendly name: title or file name. */
     QString shortName() const;
 
+    /** Get chapter index from part index. */
+    int chapterFromPart(int index);
+
+    /** Get part index from chapter index. */
+    int partFromChapter(int index);
+
+    /** Get progress (0..1) corresponding to part index and part position. */
+    qreal getProgress(int part, qreal position);
+
     QString title;                          //< Book title from EPUB.
-    QStringList toc;                        //< Table of contents from EPUB.
+    QStringList parts;                      //< EPUB part list.
     QHash<QString, ContentItem> content;    //< Content items from EPUB.
     QImage cover;                           //< Cover image.
     QStringList creators;                   //< Creators.
@@ -109,6 +115,7 @@ public:
     QString tocPath;                        //< Path to toc ncx.
     QString coverPath;                      //< Path to cover html.
     QStringList chapters;                   //< Main navigation items from EPUB.
+    qint64 size;                            //< Size of all parts.
 
 signals:
     /** Emitted if @see open() succeeds. */
@@ -118,7 +125,7 @@ protected:
     /** Extract EPUB as ZIP. */
     bool extract();
 
-    /** Parse exteacted EPUB. */
+    /** Parse extracted EPUB. */
     bool parse();
 
     /** Clear all book fields except path. */
